from leapp.libraries.common.config import get_env
from leapp import reporting
from leapp.actors import Actor
from leapp.dialogs import Dialog
from leapp.dialogs.components import BooleanComponent
from leapp.reporting import create_report, Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag
import os
import subprocess

class OSMHCheck(Actor):
    """
    Check with user that they understand limitations of managed instance upgrade.

    If admin does not confirm that they understand pre-requisites and actions needed
    to perform post-upgrade upgrade will be stalled.
    """
    name = 'alx_osmh_check'
    consumes = ()
    produces = (Report,)
    tags = (IPUWorkflowTag, ChecksPhaseTag)
    dialogs = (
        Dialog(
            scope='alx_os_management_hub_agent_check',
            reason='Confirmation',
            components=(
                BooleanComponent(
                    key='confirm',
                    label='Do you confirm that no package updates will be installed '
                          'during system upgrade with Autonomous Linux service ? ',
                    description='This system is managed by the the Autonomous Linux service '
                                'which in turn uses the OS Management Hub service to apply daily updates '
                                'and the agent is running. '
                                'Confirm that no updates will be installed during the system upgrade.',
                    reason='Installing any package during the managed instance upgrade '
                           'may leave the system in non-upgradeable state or '
                           'break the upgrade itself.'
                ),
            )
        ),
        Dialog(
            scope='alx_osmh_post_check',
            reason='Confirmation',
            components=(
                BooleanComponent(
                    key='confirm',
                    label='Do you confirm that you will wait for software sources to be attached automatically after upgrade'
                          'or you will attach Software Sources manually after the upgrade ? '
                          ' You can use the OCI console, CLI or SDK to attach software sources. ',
                    description=' Attaching software sources to the upgraded managed instance '
                                ' does not happen instantly.'
                                ' The user needs to confirm they understand'
                                ' that some actions are be needed post-upgrade',
                    reason='After upgrade, the managed instance will not have any attached software sources immediately. '
                           'If the post-upgrade steps are not performed, the upgraded instance will not have access to updates and patches.'
                ),
            )
        ),
        Dialog(
            scope='alx_os_management_hub_agent_check',
            reason='Confirmation',
            components=(
                BooleanComponent(
                    key='confirm',
                    label='Do you confirm that no package updates will be installed '
                          'during system upgrade with Autonomous Linux service, which in turn uses the OS Management Hub ? ',
                    description='This system is managed by the Autonomous Linux service '
                                'and the OS Management Hub agent is running. '
                                'Confirm that no updates will be installed during the system upgrade.',
                    reason='Installing any package during the managed instance upgrade '
                           'may leave the system in non-upgradeable state or '
                           'break the upgrade itself.'
                ),
            )
        ),
        Dialog(
            scope='alx_os_management_hub_agent_check',
            reason='Confirmation',
            components=(
                BooleanComponent(
                    key='confirm',
                    label='Do you confirm that no package updates will be installed '
                          'during system upgrade with Autonomous Linux service, which in turn uses the OS Management Hub ? ',
                    description='This system is managed by the Autonomous Linux service '
                                'but running OS Management Hub agent was not detected. '
                                'Confirm that no updates will be installed during the system upgrade.',
                    reason='Installing any package during the managed instance upgrade '
                           'may leave the system in non-upgradeable state or '
                           'break the upgrade itself.'
                ),
            )
        ),
        Dialog(
            scope='osmh_sanity_check',
            reason='Confirmation',
            components=(
                BooleanComponent(
                    key='confirm',
                    label='Do you confirm that if your instance is a previously managed by Autonomous Linux service instance, which was de-registered, '
                          ' you will not be able to register it to be managed by Autonomous Linux service after completing in-place upgrade ? ',
                    description='If system was managed by Autonomous Linux service, but was de-registered, it will not be possible to register '
                                'it again after in-place upgrade is completed. '
                                'Confirm that either instance is currently registered as Autonomous Linux managed instance before upgrade, '
                                'or you will not be registering instance to be managed by Autonomous Linux service post-upgrade.',
                    reason='Attempting to ugrade instance, that was formerly registered to be managed by Autonomous Linux service, but currently is not, '
                           'will lead to inability to register upgraded instance to be managed by Autonomous Linux service '
                ),
            )
        ),
    )

    def alx_osmh_check(self):
        try:
            ps = subprocess.Popen("ps -eaf | grep oci-osmh | grep -v grep", shell=True, stdout=subprocess.PIPE)
            output = ps.stdout.read()
            ps.stdout.close()
            ps.wait()

            if (output.find('oci-osmh') != -1):
                result = 1
            else:
                result = 0
        except:
            result = 1
        return result

    def process(self):
        osmh_plugin_status = self.alx_osmh_check()
        service_name = get_env('LEAPP_OSMH_SERVICE')
        osmh_enabled = get_env('LEAPP_OSMH')
        alx_enabled = get_env('LEAPP_ALX')
        if osmh_enabled and alx_enabled and osmh_plugin_status == 1:
            confirmations = ["alx_os_management_hub_agent_check", "alx_osmh_post_check"]
            for confirmation in confirmations:
                result = self.confirm(confirmation)
                if result and confirmation == "alx_os_management_hub_agent_check":
                    self.produce_report_cloud_agent()
                elif result and confirmation == "alx_osmh_post_check":
                    self.produce_report_osmh()
                elif result is False:
                    # user specifically chose to disagree with auto disablement
                    self.produce_inhibitor(confirmation)
        if (osmh_enabled and alx_enabled and osmh_plugin_status == 0) or (osmh_enabled and alx_enabled and not os.path.isfile("/etc/yum.repos.d/osmh.repo")):
            result = self.confirm("osmh_sanity_check")
            if result:
                self.produce_report_osmh_sanity_check()
        else:
            return

    def confirm(self, confirmations):
        service_name = get_env('LEAPP_OSMH_SERVICE')
        if service_name == "oracle-cloud-agent.service":
          dialognum=0
        elif service_name =="mgmt_agent.service":
          dialognum=2
        else:
          dialognum=3
        questions = {
            'alx_os_management_hub_agent_check': self.dialogs[dialognum],
            'alx_osmh_post_check': self.dialogs[1],
            'osmh_sanity_check': self.dialogs[4],
        }

        return self.get_answers(questions[confirmations]).get('confirm')

    def produce_report_osmh(self):
        service_name = get_env('LEAPP_OSMH_SERVICE')
        create_report([
            reporting.Title('Autonomous Linux managed instance upgrade requires user to accept certain software sources requirements.'),
            reporting.Summary(
                'User needs to confirm that they will need to wait until new software sources are automatically added, or add new software sources manually '
                'for Autonomous Linux managed instance as needed.'

            ),
            reporting.Severity(reporting.Severity.MEDIUM),
            reporting.Groups([
                    reporting.Groups.AUTHENTICATION,
                    reporting.Groups.SECURITY,
                    reporting.Groups.TOOLS
            ]),
            reporting.Remediation(hint='Do not forget to ensure accepted requirements are met'),
            reporting.RelatedResource('service', service_name)
        ])

    def produce_report_cloud_agent(self):
        service_name = get_env('LEAPP_OSMH_SERVICE')
        create_report([
            reporting.Title('Autonomous Linux managed instance upgrade requires user to accept specific package update requirements for Autonomous Linux'),
            reporting.Summary(
                ' When managed instance is being upgraded, user needs to ensure that'
                ' no packages will be installed/removed/updated via Autonomous Linux service, which in turn uses the OS Management Hub service during the upgrade'
                ' itself. '

            ),
            reporting.Severity(reporting.Severity.MEDIUM),
            reporting.Groups([
                    reporting.Groups.AUTHENTICATION,
                    reporting.Groups.SECURITY,
                    reporting.Groups.TOOLS
            ]),
            reporting.Remediation(hint='Do not forget to ensure accepted requirements are met'),
            reporting.RelatedResource('service', service_name)
        ])

    def produce_report_osmh_sanity_check(self):
        service_name = get_env('LEAPP_OSMH_SERVICE')
        create_report([
            reporting.Title('Upgrade of instance, identified as Autonomous Linux, requires user to accept specific upgrade condition requirements for Autonomous Linux'),
            reporting.Summary(
                ' When instance, idenfitied as Autonomous Linux, is being upgraded, user needs to ensure that'
                ' either instance is registered as Autonomous Linux,'
                ' or that after in-place upgrade registration will not be possible. '

            ),
            reporting.Severity(reporting.Severity.MEDIUM),
            reporting.Groups([
                    reporting.Groups.AUTHENTICATION,
                    reporting.Groups.SECURITY,
                    reporting.Groups.TOOLS
            ]),
            reporting.Remediation(hint='Do not forget to ensure accepted requirements are met'),
            reporting.RelatedResource('service', service_name)
        ])

    def produce_inhibitor(self, reason):
        service_name = get_env('LEAPP_OSMH_SERVICE')
        if reason == "alx_os_management_hub_agent_check":
            reported = "Not installing packages using Autonomous Linux service during the upgrade process"
        elif reason == "alx_osmh_post_check":
            reported = "User will need to wait or manually add/adjust profile after upgrade"
        elif reason == "osmh_sanity_check":
            reported = "Upgrade conditions of instance, reported as Autonomous Linux, are not accepted"
        create_report([
            reporting.Title(
                'Upgrade process was interrupted because user did not confirm  '
                'he complies with necessary pre or post steps, specifically - '
                '{0}.'.format(reported)),
            reporting.Summary(
                ' Upgrading managed instance requires user to ensure'
                'that packages are not installed using Autonomous Linux service, which in turn uses the OS Management Hub during the upgrade process.'
                'Also user needs to confirm that they understand that software sources will not be available immediately or are to be added manually '
                'after upgrade is finished. '
            ),
            reporting.Severity(reporting.Severity.HIGH),
            reporting.Groups([
                    reporting.Groups.AUTHENTICATION,
                    reporting.Groups.SECURITY,
                    reporting.Groups.TOOLS
            ]),
            reporting.Groups([reporting.Groups.INHIBITOR]),
            reporting.Remediation(
                hint='Confirm that you understand requirements to perform manage instance upgrade.'.format(module)),
            reporting.RelatedResource('service', '{0}')
        ])
